/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */
package cours
{
	import Box2D.Collision.Shapes.b2PolygonShape;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Dynamics.b2FixtureDef;
	import Box2D.Dynamics.b2World;

	import starling.display.DisplayObject;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;

	public class GameBox2D extends Sprite
	{
		private var bodyDef : b2BodyDef;
		
		public var m_world : b2World;
		public var m_velocityIterations : int = 10;
		public var m_positionIterations : int = 10;
		public var m_timeStep : Number = 1.0 / 30.0;

		public function GameBox2D()
		{
			// attend la fin de l'initialisation de la scène
			addEventListener(starling.events.Event.ADDED_TO_STAGE, onAdded);
		}

		private function onAdded(e : starling.events.Event) : void
		{
			// Définit le vecteur de gravité
			var gravity : b2Vec2 = new b2Vec2(0.0, 10.0);

			// Autorise le corps à dormir
			var doSleep : Boolean = true;

			// création d'un objet World
			m_world = new b2World(gravity, doSleep);

			// Variable utilisées pour créer les formes
			var body : b2Body;
			var boxShape : b2PolygonShape;
			// var circleShape : b2CircleShape;

			// Ajout d'une forme au sol
			bodyDef = new b2BodyDef();

			bodyDef.position.Set(10, 28);
			bodyDef.angle = 0.33;// angle du sol
			
			boxShape = new b2PolygonShape();
			boxShape.SetAsBox(30, 3);
			var fixtureDef : b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = boxShape;
			fixtureDef.friction = 0.3;
			
			// la forme au sol, statique, nécessite une densité de zéro
			fixtureDef.density = 0;

			// Ajout de Sprite à la forme userData
			var box : Quad = new Quad(2000, 200, 0x006f77);
			box.pivotX = box.width / 2.0;
			box.pivotY = box.height / 2.0;

			bodyDef.userData = box;
			bodyDef.userData.width = 34 * 2 * 30;
			bodyDef.userData.height = 30 * 2 * 3;
			addChild(bodyDef.userData);

			body = m_world.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);

			var quad : Quad;
			// ajout des éléments tombant au sol
			for (var i : int = 1; i < 63; i++)
			{
				bodyDef = new b2BodyDef();
				bodyDef.type = b2Body.b2_dynamicBody;
				bodyDef.position.x = Math.random() * 15 + 5;
				bodyDef.position.y = Math.random() * 10;
				var rX : Number = Math.random() + 0.5;
				var rY : Number = Math.random() + 0.5;

				// Box
				boxShape = new b2PolygonShape();
				boxShape.SetAsBox(rX, rY);
				fixtureDef.shape = boxShape;
				fixtureDef.density = 1.0;
				fixtureDef.friction = 0.5;
				fixtureDef.restitution = 0.2;

				// création d'un nouveau carré starling
				quad = new Quad(100, 100, Math.random() * 0xFFFFFF);
				quad.pivotX = quad.width / 2.0;
				quad.pivotY = quad.height / 2.0;
				
				// c'est une ligne importante - passage de userData à starling.display
				bodyDef.userData = quad;
				bodyDef.userData.width = rX * 2 * 30;
				bodyDef.userData.height = rY * 2 * 30;
				body = m_world.CreateBody(bodyDef);
				body.CreateFixture(fixtureDef);

				// affiche chaque forme (agissant comme un élément d'un corps)
				addChild(bodyDef.userData);
			}
			
			// ajout du compteur de FPS
			addChild ( new FPS() ); 

			// écouteur sur chaque frame
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}


		/**
		 * Appel à chaque image par seconde.
		 * 
		 */
		public function onEnterFrame(e : Event) : void
		{
			// mise en mouvement du monde
			m_world.Step(m_timeStep, m_velocityIterations, m_positionIterations);
			m_world.ClearForces() ;

			// mise à jour des positions / rotations du sprite dans la liste de formes
			for (var bb : b2Body = m_world.GetBodyList(); bb; bb = bb.GetNext())
			{
				// ligne importante
				// test s'il existe des objets starling.display.DisplayObject
				// et application des lois physiques 
				if (bb.GetUserData() is DisplayObject)
				{
					// cast comme un DisplayObject Starling, ce n'est pas natif !
					var sprite : DisplayObject = bb.GetUserData() as DisplayObject;
					sprite.x = bb.GetPosition().x * 13;
					sprite.y = bb.GetPosition().y * 17;
					sprite.rotation = bb.GetAngle();
				}
			}
			bodyDef.position.Set(10, 28);
		}
	}
}